==> silk-1.1.1.tar.gz: silk-1.1.1/src/rwflowpack/pdudirreader.c <==
/*
** Copyright (C) 2008 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  A "plug-in" (currently compile-time only) for rwflowpack to
**  process PDU files from a directory.
**
*/


#include "silk.h"

RCSIDENT("$SiLK: pdudirreader.c 12153 2008-08-04 15:44:20Z mwd $");

#include "sksite.h"
#include "rwflowpack_priv.h"
#include "skpolldir.h"
#include "pdusource.h"


/* LOCAL DEFINES AND TYPEDEFS */

/* A name for this reader type. */
#define READER_TYPE_NAME  "PDU Directory Reader"

typedef struct pdudir_data_st {
    skPollDirQueue_t pdq;
    skPollDir_t      polldir;
    pduSource_t      pdusource;
    uint32_t         num_recs;
    char             pathname[PATH_MAX];
    char            *filename;
} pdudir_data_t;


/* LOCAL VARIABLE DEFINITIONS */

/* The polling interval */
static uint32_t polling_interval;


/* LOCAL FUNCTION PROTOTYPES */

static int  readerSetup(
    fp_daemon_mode_t   *is_daemon,
    const sk_vector_t  *probe_vec,
    reader_options_t   *options);
static void readerTeardown(void);
static int  readerStart(flow_proc_t *fproc);
static int  readerStop(flow_proc_t *fproc);
static fp_get_record_result_t readerGetRecord(
    rwRec                  *out_rwrec,
    const skpc_probe_t    **out_probe,
    flow_proc_t            *fproc);
static int  readerWantProbe(skpc_probe_t *probe);


/* FUNCTION DEFINITIONS */

/*
 *  ok = readerStart(flow_processor)
 *
 *    Create a flowsource object that will read records from PDU files
 *    in a directory.
 *
 *    Invoked by reader_type->start_fn();
 */
static int readerStart(flow_proc_t *fproc)
{
    const char *dir;
    pdudir_data_t *pdudir_source;

    /* Should be NULL when starting */
    assert(fproc->flow_src == NULL);

    dir = skpcProbeGetPollDirectory(fproc->probe);
    assert(dir != NULL);

    pdudir_source = (pdudir_data_t *)calloc(1, sizeof(*pdudir_source));
    if (pdudir_source == NULL) {
        /* memory error.  Fail */
        ERRMSG("Out of memory error");
        return -1;
    }

    pdudir_source->pdq = skPollDirQueueCreate();
    if (pdudir_source->pdq == NULL) {
        /* memory error.  Fail */
        ERRMSG("Could not create polldir queue");
        return -1;
    }

    pdudir_source->polldir =
        skPollDirCreate(dir, polling_interval, pdudir_source->pdq);

    if (pdudir_source->polldir == NULL) {
        skPollDirQueueDestroy(pdudir_source->pdq);
        ERRMSG("Could not initiate polling on directory %s", dir);
        return -1;
    }

    fproc->flow_src = pdudir_source;

    return 0;
}


/*
 *  readerStop();
 *
 *    Stop reading records.
 *
 *    Invoked by reader_type->stop_fn();
 */
static int readerStop(flow_proc_t *fproc)
{
    pdudir_data_t *pdudir_source = (pdudir_data_t *)fproc->flow_src;

    if (pdudir_source) {
        if (pdudir_source->pdusource) {
            pduSourceDestroy(pdudir_source->pdusource);
        }
        if (pdudir_source->polldir) {
            skPollDirDestroy(pdudir_source->polldir);
        }
        if (pdudir_source->pdq) {
            skPollDirQueueDestroy(pdudir_source->pdq);
        }
        free(pdudir_source);
        fproc->flow_src = NULL;
    }
    return 0;
}


/*
 *  readerPrintStats(fproc);
 *
 *    Print to the log the statistics associated with the flow
 *    processor 'proc'.
 */
static void readerPrintStats(const flow_proc_t *fproc)
{
    pdudir_data_t *pdudir_source = (pdudir_data_t *)fproc->flow_src;
    flowsource_stats_t stats;

    pduSourceGetStats(pdudir_source->pdusource, &stats);
    INFOMSG(("%s: Pkts %10" PRIu64 "/%" PRIu64
             " Recs %10" PRIu64 " MissRecs %10" PRId64 " BadRecs %10" PRIu64),
            pdudir_source->filename, (stats.procPkts - stats.badPkts),
            stats.procPkts, stats.goodRecs, stats.missingRecs,
            (stats.badRecs + fproc->rec_count_bad));
}


/*
 *  readerGetNextValidFile();
 *
 *    Pull the next file name off of the valid-queue and create a
 *    flowsource object to read the flowcap records in it.  Fills
 *    'fproc' with the new pdu-source object.
 *
 *    Return 0 on success.  Return -1 if getting the file name fails;
 *    return -2 if unable to open the file or file not of correct
 *    form.
 */
static int readerGetNextValidFile(flow_proc_t *fproc)
{
    int rv;
    pdudir_data_t *pdudir_source = (pdudir_data_t *)fproc->flow_src;

    /* Get next file from the directory poller */
    rv = skPollDirGetNextFile(pdudir_source->pdq, pdudir_source->pathname,
                              &pdudir_source->filename);
    if (rv != 0) {
        if ((rv == -1) &&
            (skPollDirGetError(pdudir_source->polldir) != PDERR_NONE))
        {
            CRITMSG("Polldir error ocurred at %s:%d",
                    __FILE__, __LINE__);
            abort();
        }
        return -1;
    }

    INFOMSG((READER_TYPE_NAME " processing %s"), pdudir_source->filename);

    /* Zero record count */
    pdudir_source->num_recs = 0;

    /* open the file to create a source of records */
    assert(pdudir_source->pdusource == NULL);
    pdudir_source->pdusource = pduFileSourceCreate(pdudir_source->pathname);
    if (pdudir_source->pdusource == NULL) {
        return -2;
    }

    assert(fproc->probe != NULL);

    return 0;
}


/*
 *  readerGetRecord(&out_rwrec, &out_probe);
 *
 *    Fill 'out_rwrec' with an rw generic record from the underlying
 *    flowsource object.  If there is no flowsource object, create one
 *    by pulling a file off the queue.  Fill 'out_probe' with the
 *    probe where the flow was collected.
 *
 *    Invoked by reader_type->get_record_fn();
 */
fp_get_record_result_t readerGetRecord(
    rwRec                  *out_rwrec,
    const skpc_probe_t    **out_probe,
    flow_proc_t            *fproc)
{
    pdudir_data_t *pdudir_source = (pdudir_data_t *)fproc->flow_src;
    fp_get_record_result_t retVal = FP_GET_ERROR;

    /* If we don't have a source, pop a file off the valid-queue
     * and start processing it. */
    if (pdudir_source->pdusource == NULL) {
        switch (readerGetNextValidFile(fproc)) {
          case 0:
            /* Success */
            break;
          case -1:
            /* Error getting file name (maybe in shutdown?) */
            goto END;
          case -2:  /* Error opening file */
          default:  /* Unexpected value */
            retVal = FP_FATAL_ERROR;
            goto END;
        }
    }

    /* Assume we can get a record from the probe. */
    retVal = FP_RECORD;
    *out_probe = fproc->probe;

    /* Try to get the record */
    if (pduSourceGetGeneric(pdudir_source->pdusource, out_rwrec)) {
        /* get failed: either at EOF or got an error; assume EOF. */
        retVal = FP_FILE_BREAK;
        *out_probe = NULL;

        readerPrintStats(fproc);

        /* Clear out, getting ready for next file */
        pduSourceDestroy(pdudir_source->pdusource);
        pdudir_source->pdusource = NULL;

        /* Either archive the file or remove it */
        if (archiveFile(pdudir_source->pathname) == 1) {
            /* archiving not requested */
            if (unlink(pdudir_source->pathname) == -1) {
                WARNINGMSG("Couldn't remove %s: %s",
                           pdudir_source->filename, strerror(errno));
            }
        }
    } else {
        pdudir_source->num_recs++;
    }

  END:
    return retVal;
}

/*
 * pduDirReaderInitialize:
 *
 *    Set initial state of this flow reader.
 *
 *    Invoked by reader_type->initialize_fn();
 */
int pduDirReaderInitialize(reader_type_t *reader_type)
{
    /* Set my name */
    reader_type->reader_name = READER_TYPE_NAME;

    /* Set function pointers */
    reader_type->setup_fn            = &readerSetup;
    reader_type->teardown_fn         = &readerTeardown;
    reader_type->start_fn            = &readerStart;
    reader_type->stop_fn             = &readerStop;
    reader_type->get_record_fn       = &readerGetRecord;
    reader_type->want_probe_fn       = &readerWantProbe;

    return 0;
}


/*
 *  yes_or_no = readerWantProbe(probe);
 *
 *    Return a TRUE value this reader_type is able to process the data
 *    from the 'probe'; return a FALSE value otherwise.
 *
 *    Invoked by reader_type->want_probe_fn();
 */
static int readerWantProbe(skpc_probe_t *probe)
{
    /* This is what we expect: a directory based NetFlow v5 listener */
    if ((skpcProbeGetPollDirectory(probe) != NULL)
        && (PROBE_ENUM_NETFLOW_V5 == skpcProbeGetType(probe)))
    {
        return 1;
    }

    return 0;
}



/*
 *  status = readerSetup(&out_daemon_mode, probe_vector, options);
 *
 *    The reader_type should do any setup it requires prior to
 *    starting its flow_processor instances: Validate and/or store the
 *    reader_type's 'options'.  'probe_vector' is the list of probes
 *    that this reader_type said it would process
 *    (readerWantProbe).  'out_daemon_mode' should be set according
 *    to how this reader_type expects to run.
 *
 *    Invoked by reader_type->setup_fn();
 */
static int readerSetup(
    fp_daemon_mode_t   *is_daemon,
    const sk_vector_t  *probe_vec,
    reader_options_t   *options)
{
    size_t probe_count;

    /* this function should only be called if we actually have probes
     * to process */
    probe_count = skVectorGetCount(probe_vec);
    if (0 == probe_count) {
        return 1;
    }

    polling_interval = options->pdu_dir.polling_interval;

    /* Is a daemon */
    *is_daemon = FP_DAEMON_ON;

    return 0;
}

/*
 *  readerTeardown();
 *
 *    Destroy the PDU dir reader.
 *
 *    Invoked by reader_type->teardown_fn();
 */
static void readerTeardown(void)
{
    /* nothing to do */
}


/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/

==> silk-1.1.3.tar.gz: silk-1.1.3/src/rwflowpack/pdudirreader.c <==
/*
** Copyright (C) 2008 by Carnegie Mellon University.
**
** @OPENSOURCE_HEADER_START@
**
** Use of the SILK system and related source code is subject to the terms
** of the following licenses:
**
** GNU Public License (GPL) Rights pursuant to Version 2, June 1991
** Government Purpose License Rights (GPLR) pursuant to DFARS 252.225-7013
**
** NO WARRANTY
**
** ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
** PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
** PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
** "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
** KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
** LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
** MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
** OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
** SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
** TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
** WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
** LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
** CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
** CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
** DELIVERABLES UNDER THIS LICENSE.
**
** Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
** Mellon University, its trustees, officers, employees, and agents from
** all claims or demands made against them (and any related losses,
** expenses, or attorney's fees) arising out of, or relating to Licensee's
** and/or its sub licensees' negligent use or willful misuse of or
** negligent conduct or willful misconduct regarding the Software,
** facilities, or other rights or assistance granted by Carnegie Mellon
** University under this License, including, but not limited to, any
** claims of product liability, personal injury, death, damage to
** property, or violation of any laws or regulations.
**
** Carnegie Mellon University Software Engineering Institute authored
** documents are sponsored by the U.S. Department of Defense under
** Contract F19628-00-C-0003. Carnegie Mellon University retains
** copyrights in all material produced under this contract. The U.S.
** Government retains a non-exclusive, royalty-free license to publish or
** reproduce these documents, or allow others to do so, for U.S.
** Government purposes only pursuant to the copyright license under the
** contract clause at 252.227.7013.
**
** @OPENSOURCE_HEADER_END@
*/

/*
**  A "plug-in" (currently compile-time only) for rwflowpack to
**  process PDU files from a directory.
**
*/


#include "silk.h"

RCSIDENT("$SiLK: pdudirreader.c 12526 2008-09-18 16:09:29Z mthomas $");

#include "rwflowpack_priv.h"
#include "skpolldir.h"
#include "pdusource.h"
#include "skdeque.h"


/* LOCAL DEFINES AND TYPEDEFS */

/* A name for this reader type. */
#define READER_TYPE_NAME  "PDU Directory Reader"

typedef struct pdudir_data_st {
    char             pathname[PATH_MAX];
    char            *filename;
    skPollDirQueue_t pdq;
    skPollDir_t      polldir;
    pduSource_t      pdusource;
} pdudir_data_t;


/* LOCAL VARIABLE DEFINITIONS */

/* The polling interval */
static uint32_t polling_interval;

/* List of pdudir_data_t objects saved here for shutdown purposes */
static skDeque_t pdudir_list = NULL;

/* LOCAL FUNCTION PROTOTYPES */

static int  readerSetup(
    fp_daemon_mode_t   *is_daemon,
    const sk_vector_t  *probe_vec,
    reader_options_t   *options);
static int  readerStart(flow_proc_t *fproc);
static int  readerStop(flow_proc_t *fproc);
static fp_get_record_result_t readerGetRecord(
    rwRec                  *out_rwrec,
    const skpc_probe_t    **out_probe,
    flow_proc_t            *fproc);
static int  readerWantProbe(skpc_probe_t *probe);
static void readerTeardown(void);

/* FUNCTION DEFINITIONS */

/*
 *  ok = readerStart(flow_processor)
 *
 *    Create a flowsource object that will read records from PDU files
 *    in a directory.
 *
 *    Invoked by reader_type->start_fn();
 */
static int readerStart(flow_proc_t *fproc)
{
    const char *dir;
    pdudir_data_t *pdudir_source;

    /* Should be NULL when starting */
    assert(fproc->flow_src == NULL);

    dir = skpcProbeGetPollDirectory(fproc->probe);
    assert(dir != NULL);

    pdudir_source = (pdudir_data_t *)calloc(1, sizeof(*pdudir_source));
    if (pdudir_source == NULL) {
        /* memory error.  Fail */
        ERRMSG("Out of memory error");
        return -1;
    }

    /* Set up directory polling */
    DEBUGMSG(("Creating " READER_TYPE_NAME
              " directory poller for probe %s on '%s'"),
             skpcProbeGetName(fproc->probe), dir);

    pdudir_source->pdq = skPollDirQueueCreate();
    if (pdudir_source->pdq == NULL) {
        /* memory error.  Fail */
        free(pdudir_source);
        ERRMSG("Could not create polldir queue");
        return -1;
    }

    pdudir_source->polldir = skPollDirCreate(dir, polling_interval,
                                             pdudir_source->pdq);
    if (pdudir_source->polldir == NULL) {
        skPollDirQueueDestroy(pdudir_source->pdq);
        free(pdudir_source);
        ERRMSG("Could not initiate polling on directory %s", dir);
        return -1;
    }

    if (skDequePushBack(pdudir_list, pdudir_source) != SKDQ_SUCCESS) {
        skPollDirQueueDestroy(pdudir_source->pdq);
        skPollDirDestroy(pdudir_source->polldir);
        free(pdudir_source);
        ERRMSG("Could not add pdudir source to queue");
        return -1;
    }

    fproc->flow_src = pdudir_source;

    return 0;
}


/*
 *  readerStop();
 *
 *    Stop reading records.
 *
 *    Invoked by reader_type->stop_fn();
 */
static int readerStop(flow_proc_t *fproc)
{
    pdudir_data_t *pdudir_source = (pdudir_data_t *)fproc->flow_src;

    if (pdudir_source) {
        if (pdudir_source->pdusource) {
            pduSourceStop(pdudir_source->pdusource);
        }
        if (pdudir_source->polldir) {
            DEBUGMSG("Destroying directory poller");
            skPollDirDestroy(pdudir_source->polldir);
        }
        if (pdudir_source->pdq) {
            skPollDirQueueDestroy(pdudir_source->pdq);
        }
    }

    return 0;
}

/*
 *  readerTeardown();
 *
 *    Destroy the PDU reader.
 *
 *    Invoked by reader_type->teardown_fn();
 */
static void readerTeardown(void)
{
    pdudir_data_t *pdudir_source;

    if (pdudir_list) {
        while (skDequePopFrontNB(pdudir_list, (void **)&pdudir_source)
               == SKDQ_SUCCESS)
        {
            if (pdudir_source->pdusource) {
                pduSourceDestroy(pdudir_source->pdusource);
            }
            free(pdudir_source);
        }
        skDequeDestroy(pdudir_list);
        pdudir_list = NULL;
    }
}


/*
 *  readerPrintStats(fproc);
 *
 *    Print to the log the statistics associated with the flow
 *    processor 'proc'.
 */
static void readerPrintStats(const flow_proc_t *fproc)
{
    pdudir_data_t *pdudir_source = (pdudir_data_t *)fproc->flow_src;
    flowsource_stats_t stats;

    pduSourceGetStats(pdudir_source->pdusource, &stats);
    INFOMSG(("%s: Pkts %10" PRIu64 "/%" PRIu64
             " Recs %10" PRIu64 " MissRecs %10" PRId64 " BadRecs %10" PRIu64),
            pdudir_source->filename, (stats.procPkts - stats.badPkts),
            stats.procPkts, stats.goodRecs, stats.missingRecs,
            (stats.badRecs + fproc->rec_count_bad));
}


/*
 *  readerGetNextValidFile(fproc);
 *
 *    Pull the next file name from the poll-directory and create a
 *    flowsource object to read the PDU records in it.  Modifies the
 *    source object that 'fproc' contains.
 *
 *    Return 0 on success.  Return -1 if getting the file name fails.
 *    If unable to open the file or file not of correct form, return
 *    -2 unless the --error-dir is set, in which case move the file
 *    there and try the next file.
 */
static int readerGetNextValidFile(flow_proc_t *fproc)
{
    int rv;
    pdudir_data_t *pdudir_source = (pdudir_data_t *)fproc->flow_src;

    do {
        /* Get next file from the directory poller */
        rv = skPollDirGetNextFile(pdudir_source->pdq, pdudir_source->pathname,
                                  &pdudir_source->filename);
        if (rv != 0) {
            if ((rv == -1) &&
                (skPollDirGetError(pdudir_source->polldir) != PDERR_NONE))
            {
                CRITMSG("Polldir error ocurred at %s:%d",
                        __FILE__, __LINE__);
                abort();
            }
            return -1;
        }

        INFOMSG((READER_TYPE_NAME " processing %s"), pdudir_source->filename);

        /* Zero record count */
        fproc->rec_count_total = 0;
        fproc->rec_count_bad = 0;
        assert(pdudir_source->pdusource == NULL);

        /* open the file to create a source of records */
        pdudir_source->pdusource =pduFileSourceCreate(pdudir_source->pathname);
        if (pdudir_source->pdusource == NULL) {
            rv = moveToErrorDir(pdudir_source->pathname);
            if (rv != 0) {
                /* either no --error-dir (rv == 1) or problem moving
                 * the file (rv == -1).  either way, return an error
                 * code to the caller. */
                return -2;
            }
        }
    } while (pdudir_source->pdusource == NULL);

    assert(fproc->probe != NULL);

    return 0;
}


/*
 *  readerGetRecord(&out_rwrec, &out_probe);
 *
 *    Fill 'out_rwrec' with an rw generic record from the underlying
 *    flowsource object.  If there is no flowsource object, create one
 *    by pulling a file off the queue.  Fill 'out_probe' with the
 *    probe where the flow was collected.
 *
 *    Invoked by reader_type->get_record_fn();
 */
fp_get_record_result_t readerGetRecord(
    rwRec                  *out_rwrec,
    const skpc_probe_t    **out_probe,
    flow_proc_t            *fproc)
{
    pdudir_data_t *pdudir_source = (pdudir_data_t *)fproc->flow_src;
    fp_get_record_result_t retVal = FP_GET_ERROR;

    /* If we don't have a source, pop a file off the valid-queue
     * and start processing it. */
    if (pdudir_source->pdusource == NULL) {
        switch (readerGetNextValidFile(fproc)) {
          case 0:
            /* Success */
            break;
          case -1:
            /* Error getting file name (maybe in shutdown?) */
            goto END;
          case -2:  /* Error opening file */
          default:  /* Unexpected value */
            retVal = FP_FATAL_ERROR;
            goto END;
        }
    }

    /* Assume we can get a record from the probe. */
    retVal = FP_RECORD;
    *out_probe = fproc->probe;

    /* Try to get a record */
    if (-1 == pduSourceGetGeneric(pdudir_source->pdusource, out_rwrec)) {
        /* Failed to get a record: either at EOF or got an error */

        if (0 == fproc->rec_count_total) {
            /* failed to get first record.  treat entire file as bad */
            NOTICEMSG("File '%s' does not appear to be a NetFlow v5 file",
                      pdudir_source->pathname);

            if (0 == moveToErrorDir(pdudir_source->pathname)) {
                /* moved file to error-dir */
                retVal = FP_FILE_BREAK;
            } else {
                /* either no --error-dir or problem moving the file.
                 * either way, return an error code to the caller. */
                retVal = FP_FATAL_ERROR;
            }
        } else {
            /* we've seen good records in the past, so assume EOF. */
            retVal = FP_FILE_BREAK;

            readerPrintStats(fproc);

            /* Either archive the file or remove it */
            if (archiveFile(pdudir_source->pathname) == 1) {
                /* archiving not requested */
                if (unlink(pdudir_source->pathname) == -1) {
                    WARNINGMSG("Could not remove %s: %s",
                               pdudir_source->filename, strerror(errno));
                }
            }
        }

        /* Clear out, getting ready for next file */
        *out_probe = NULL;
        pduSourceDestroy(pdudir_source->pdusource);
        pdudir_source->pdusource = NULL;
    }

  END:
    return retVal;
}

/*
 * pduDirReaderInitialize:
 *
 *    Set initial state of this flow reader.
 *
 *    Invoked by reader_type->initialize_fn();
 */
int pduDirReaderInitialize(reader_type_t *reader_type)
{
    /* Set my name */
    reader_type->reader_name = READER_TYPE_NAME;

    /* Set function pointers */
    reader_type->setup_fn            = &readerSetup;
    reader_type->teardown_fn         = &readerTeardown;
    reader_type->start_fn            = &readerStart;
    reader_type->stop_fn             = &readerStop;
    reader_type->get_record_fn       = &readerGetRecord;
    reader_type->want_probe_fn       = &readerWantProbe;

    return 0;
}


/*
 *  yes_or_no = readerWantProbe(probe);
 *
 *    Return a TRUE value this reader_type is able to process the data
 *    from the 'probe'; return a FALSE value otherwise.
 *
 *    Invoked by reader_type->want_probe_fn();
 */
static int readerWantProbe(skpc_probe_t *probe)
{
    /* This is what we expect: a directory based NetFlow v5 listener */
    if ((skpcProbeGetPollDirectory(probe) != NULL)
        && (PROBE_ENUM_NETFLOW_V5 == skpcProbeGetType(probe)))
    {
        return 1;
    }

    return 0;
}



/*
 *  status = readerSetup(&out_daemon_mode, probe_vector, options);
 *
 *    The reader_type should do any setup it requires prior to
 *    starting its flow_processor instances: Validate and/or store the
 *    reader_type's 'options'.  'probe_vector' is the list of probes
 *    that this reader_type said it would process
 *    (readerWantProbe).  'out_daemon_mode' should be set according
 *    to how this reader_type expects to run.
 *
 *    Invoked by reader_type->setup_fn();
 */
static int readerSetup(
    fp_daemon_mode_t   *is_daemon,
    const sk_vector_t  *probe_vec,
    reader_options_t   *options)
{
    /* this function should only be called if we actually have probes
     * to process */
    if (0 == skVectorGetCount(probe_vec)) {
        return 1;
    }

    pdudir_list = skDequeCreate();
    if (pdudir_list == NULL) {
        return 1;
    }

    polling_interval = options->pdu_dir.polling_interval;

    /* Is a daemon */
    *is_daemon = FP_DAEMON_ON;

    return 0;
}


/*
** Local Variables:
** mode:c
** indent-tabs-mode:nil
** c-basic-offset:4
** End:
*/
